import os, sys, math, random
import pygame

class EventListener:
    def processevent(self,event):
        pass

class MainManager:
    def __init__(self,width=800,height=600,desiredfps=60):
        pygame.init
        self.quitting = 0
        self.desiredfps = desiredfps
        self.frametime = 1/float(desiredfps)
        self.clock = pygame.time.Clock()
        self.width = width
        self.height = height
        self.screen = pygame.display.set_mode((self.width,self.height))
        
        self.listeners = []
        
        pygame.mixer.init()
        self.soundchannel = pygame.mixer.Channel(0)
        
    def addeventlistener(self,listener):
        self.listeners.append(listener)
    
    def processEvent(self,event):
        if event.type == pygame.QUIT:
            self.quitting = 1
            return
        if (event.type == pygame.KEYDOWN) and (event.key == pygame.K_ESCAPE):
            self.quitting = 1
            return
        for listener in self.listeners:
            listener.processevent(event)

class GameLoop:
    def __init__(self):
        self.loopdone = 0
        
    def quitloop(self):
        self.loopdone = 1        
         
    def update(self):
        pass       
        
    def run(self):
        while not(self.loopdone) and not(manager.quitting):
            for event in pygame.event.get():
                manager.processEvent(event) 
            manager.clock.tick(manager.desiredfps)
            self.update()
            pygame.display.flip()

def sign(r):
    if r>0:
        return 1
    if r<0:
        return -1
    return 0
        
class Vector:
        
    def __init__(self,x=0,y=0):
        self.x = x
        self.y = y
        
    def copy(self):
        return Vector(self.x,self.y)
    
    def magnitude(self):
        return math.sqrt(self.x*self.x+self.y*self.y)
    
    def normalise(self):
        length = self.magnitude()
        self.x = self.x/length
        self.y = self.y/length
    
    def swapaxis(self):
        temp = self.x
        self.x = self.y
        self.y = temp
        
    def __add__(self,v):
        return Vector(self.x+v.x,self.y+v.y)
    
    def __iadd__(self,v):
        self.x += v.x
        self.y += v.y
    
    def __sub__(self,v):
        return Vector(self.x-v.x,self.y-v.y)
    
    def __mul__(self,r):
        return Vector(self.x*r,self.y*r)
    
    def __div__(self,r):
        return Vector(self.x/r,self.y/r)
    
    def getside(self):
        if (self.x == 0) and (self.y == 0):
            return 'n'
        if self.x == 0:
            if self.y>0:
                return 't'
            else:
                return 'b'
        if self.y == 0:
            if self.x>0:
                return 'l'
            else:
                return 'r'
        if abs(self.x) < abs(self.y):
            if self.x>0:
                return 'l'
            else:
                return 'r'
        else:
            if self.y>0:
                return 't'
            else:
                return 'b'

class LineSegment:
    def __init__(self,svec,evec):
        self.svec = svec
        self.evec = evec
        
    def length(self):
        tempvec = self.evec-self.svec
        return tempvec.magnitude()
        
    def direction(self):
        return self.evec-self.svec
    
    def normaldirection(self):
        return self.direction/self.length

def project2D(direction,offset):
    ratio = offset/direction.y;
    return ratio*direction.x;

def between(test,bound1,bound2):
    upper = max(bound1,bound2)
    lower = min(bound1,bound2)

    return (test>=lower) and (test<=upper)

class CustomRect:
    def __init__(self,centre,extents):
        self.centre = centre
        self.extents = extents
        self.rect = pygame.Rect(centre.x-extents.x,centre.y-extents.y,extents.x*2,extents.y*2)
        
    def draw(self,surface,colour):
        pygame.draw.rect(surface,colour,self.rect,0)
        
    def drawimage(self,surface,image):
        drawimage = image[0]
        imagerect = image[1]
        sourcearea = pygame.Rect(0,0,self.rect.width,self.rect.height)            
        surface.blit(drawimage,self.rect,sourcearea) 
        
    def drawtiledimage(self,surface,image):
        drawimage = image[0]
        imagerect = image[1]
        if (self.rect.width<=imagerect.width) and (self.rect.height<=imagerect.height):
            sourcearea = pygame.Rect(0,0,self.rect.width,self.rect.height)            
            surface.blit(drawimage,self.rect,sourcearea) 
            return
        drawrect = pygame.Rect(self.rect.left,self.rect.top,imagerect.width,imagerect.height)
        testrect = self.rect
        while drawrect.left<testrect.right:
            drawrect.top = self.rect.top
            sourcearea = pygame.Rect(0,0,testrect.right-drawrect.left,testrect.bottom-drawrect.top)
            surface.blit(drawimage,drawrect,sourcearea)
            while drawrect.top<testrect.bottom:
                sourcearea = pygame.Rect(0,0,testrect.right-drawrect.left,testrect.bottom-drawrect.top)
                surface.blit(drawimage,drawrect,sourcearea)
                drawrect.top = drawrect.bottom
            drawrect.left = drawrect.right
        
    def colliderect(self,r):
        return self.rect.colliderect(r.rect)
    
    def getoverlap(self,r):
        result = Vector()
        
        difference = r.centre-self.centre
        
        if self.extents.x+r.extents.x < abs(difference.x):
            return result
        if self.extents.y+r.extents.y < abs(difference.y):
            return result
        
        result.x = (self.extents.x+r.extents.x-abs(difference.x))*sign(difference.x)
        result.y = (self.extents.y+r.extents.y-abs(difference.y))*sign(difference.y)
        return result
    
    def lineintersect(self,ls):        
        tl = self.centre-self.extents
        br = self.centre+self.extents

        direction = ls.direction()

        #Test top edge
        if (direction.y>0) and (ls.svec.y<tl.y):
            offset = project2D(direction,tl.y-ls.svec.y)
            if between(ls.svec.x+offset,tl.x,br.x):
                if between(offset,0,ls.evec.x-ls.svec.x):
                    collided = 1
                    colpoint= Vector(ls.svec.x+offset,tl.y)
                    return (collided,colpoint)
    
        #Test bottom edge
        if (direction.y<0) and (ls.svec.y>br.y):
            offset = project2D(direction,br.y-ls.svec.y)
            if between(ls.svec.x+offset,tl.x,br.x):
                if between(offset,0,ls.evec.x-ls.svec.x):
                    collided = 1
                    colpoint = Vector(ls.svec.x+offset,br.y)
                    return (collided,colpoint)

        #Test left edge
        if (direction.x>0) and (ls.svec.x<tl.x):
            tempdir = direction
            tempdir.swapaxis()
            offset = project2D(tempdir,tl.x-ls.svec.x)
            if between(ls.svec.y+offset,tl.y,br.y):
                if between(offset,0,ls.evec.y-ls.svec.y):
                    collided = 1
                    colpoint = Vector(tl.x,ls.svec.y+offset)
                    return(collided,colpoint)

        #Test right edge
        if (direction.x<0) and (ls.svec.x>br.x):
            tempdir = direction
            tempdir.swapaxis()
            offset = project2D(tempdir,br.x-ls.svec.x)
            if between(ls.svec.y+offset,tl.y,br.y):
                if between(offset,0,ls.evec.y-ls.svec.y):
                    collided = 1
                    colpoint = Vector(br.x,ls.svec.y+offset);
                    return(collided,colpoint)
        return(0,Vector(0,0))

class Block:
    def __init__(self,tl=Vector(),br=Vector()):
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)
        self.colour = (0,0,128)
        self.image = resources.blockimage
        
    def makefromstring(self,line):
        line = line[line.find(',')+1:]
        x0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        x1 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y1 = int(line)
        
        tl = Vector(x0,y0)
        br = Vector(x1,y1)
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)      
        
    def colliderect(self,r):
        return self.customrect.colliderect(r)
    
    def getoverlap(self,r):
        return self.customrect.getoverlap(r)    
    
    def lineintersect(self,ls):
        return self.customrect.lineintersect(ls)
    
    def reactactor(self,actor):
        pass

class GameActor:
    def __init__(self,xpos,ypos,xextents=10,yextents=20,isplayer=0):
        self.position = Vector(xpos,ypos)
        self.startpos = self.position
        self.velocity = Vector(0,0)
        self.acceleration = Vector(0,1200)
        self.canjump = 0
        self.jumping = 0
        self.isplayer = isplayer
        
        self.faceleft = 0
        self.currentframe = 0
        self.frametime = 0
        
        self.extents = Vector(xextents,yextents)
        
    def move(self):
        self.velocity = self.velocity+(self.acceleration*manager.frametime)
        self.position = self.position+(self.velocity*manager.frametime)
        
        if len(resources.guyanim)>0:
            if (self.canjump) and (self.velocity.x <> 0):
                self.frametime += manager.frametime
            if (self.canjump) and (self.velocity.x == 0):
                self.frametime = 0
                self.currentframe = 0
            if self.frametime>0.05:
                self.frametime = 0
                self.currentframe += 1
                if self.currentframe == len(resources.guyanim):
                    self.currentframe = 0                
        
    def getrect(self):
        return CustomRect(self.position.copy(),self.extents.copy())
    
    def testblock(self,block):
        testrect = CustomRect(self.position,self.extents)
        if block.colliderect(testrect):
            overlap = block.getoverlap(testrect)*-1
            side = overlap.getside()
            if side == 't':
                self.canjump = 1
            if side in ('t','b'):
                self.position.y -= overlap.y
                self.velocity.y = 0
            if side in ('l','r'):
                self.position.x -= overlap.x
                self.velocity.x = 0
            block.reactactor(self)
    
    def collide(self,blocks):
        self.canjump = 0
        for block in blocks:
            self.testblock(block)
        
    def draw(self,colour):
        drawrect = CustomRect(self.position,self.extents)
        
        drawrect.draw(manager.screen,colour)
        
    def attemptjump(self):
        if (self.canjump) and not(self.jumping):
            self.canjump = 0
            self.jumping = 1
            self.velocity.y = -500
            
            resources.jumpsound.play()
            
class Projectile:
    def __init__(self,room,position,velocity):
        self.room = room
        self.position = position.copy()
        self.oldpos = self.position
        self.velocity = velocity.copy()
        self.acceleration = Vector(0,1200)
        
    def move(self):
        self.oldpos = self.position
        self.velocity = self.velocity+(self.acceleration*manager.frametime)
        self.position = self.position+(self.velocity*manager.frametime)
    
    def testblock(self,block):
        testline = LineSegment(self.oldpos,self.position)
        intersectresult = block.lineintersect(testline)
        collided = intersectresult[0]
        if collided:            
            return 1
        return 0
    
    def collide(self):
        for block in self.room.blocks:
            if self.testblock(block):
                self.die()
                return
        
    def draw(self,colour,camera):
        svec = Vector(self.oldpos.x,self.oldpos.y)
        svec = svec-camera
        evec = Vector(self.position.x,self.position.y)
        evec = evec-camera
        pygame.draw.aaline(manager.screen,colour,(svec.x,svec.y),(evec.x,evec.y),1)
        
    def die(self):
        self.room.deadbullets.append(self)

def loadimage(path,colourkey=-1):
    actualpath = os.path.join('images',path)
    image = pygame.image.load(actualpath)
    if colourkey>-1:
        image.set_colorkey(colourkey)
    else:
        image.set_colorkey(image.get_at((0,0)))
    image.convert()
    return (image,image.get_rect())

def loadsound(path):
    actualpath = os.path.join('sounds',path)
    sound = pygame.mixer.Sound(actualpath)
    return sound

class InputWaiter(EventListener):
    def __init__(self):
        self.waiting = 1
        self.allowwait = 0
        
    def processevent(self,event):
        if event.type in (pygame.KEYDOWN,pygame.MOUSEBUTTONDOWN):
            if self.allowwait:
                self.waiting = 0

class ImageViewerLoop(GameLoop):
    def __init__(self,imagepath,nextloop):
        GameLoop.__init__(self)
        self.waiter = InputWaiter()
        manager.addeventlistener(self.waiter)
        self.time = 0
        loadedimage = loadimage(imagepath)
        self.image = loadedimage[0]
        loadedrect = loadedimage[1]
        posx = (manager.width-loadedrect.width)/2
        posy = (manager.height-loadedrect.height)/2
        self.imagepos = pygame.Rect(posx,posy,0,0)
        self.nextloop = nextloop
        
    def update(self):
        self.waiter.allowwait = 1
        manager.screen.fill((0,0,0))
        manager.screen.blit(self.image,self.imagepos)
        
        self.time += manager.frametime
        if self.time>0.25:
            if not(self.waiter.waiting):
                manager.listeners.remove(self.waiter)
                self.quitloop()
                self.nextloop.run()
    
########################### END GLOBAL DEFINITIONS  ##########################

LEFT_KEY = 0
RIGHT_KEY = 1
JUMP_KEY = 2

class KeyState:
    def __init__(self,value,state=0):
        self.value = value
        self.state = state

class Controls(EventListener):
    def __init__(self,keylist):
        self.keylist = []
        for key in keylist:
            self.keylist.append(KeyState(key))
        
        self.mouseabsx = 0
        self.mouseabsy = 0
        
        self.mousepos = Vector()
        self.mouseworldpos = Vector()
        self.mouseleft = 0
        self.mouseright = 0
        
    def processevent(self,event):
        if event.type == pygame.KEYDOWN:
            for key in self.keylist:
                if key.value == event.key:
                    key.state = 1
        if event.type == pygame.KEYUP:
            for key in self.keylist:
                if key.value == event.key:
                    key.state = 0
        if event.type == pygame.MOUSEMOTION:
            self.mouseabsx = event.pos[0]
            self.mouseabsy = event.pos[1]
            self.mousepos.x = event.pos[0]
            self.mousepos.y = event.pos[1]
        if event.type == pygame.MOUSEBUTTONDOWN:
            if event.button == 1:
                self.mouseleft = 1
            if event.button == 3:
                self.mouseright == 1
        if event.type == pygame.MOUSEBUTTONUP:
            if event.button == 1:
                self.mouseleft = 0
            if event.button == 3:
                self.mouseright == 0

class Weapon:
    def __init__(self,semiauto,firetime,clipmax,cliptime):
        self.semiauto = semiauto
        self.firetime = firetime
        self.firereload = 0
        self.clipmax = clipmax
        self.clipleft = self.clipmax
        self.cliptime = cliptime
        self.reloading = 0
        self.clipreload = 0
        self.canfire = 0
        
    def update(self,dtime):
        if self.firereload >0:
            self.firereload -= dtime
            
        if self.reloading:
            self.clipreload -= dtime
            if self.clipreload < 0:
                self.clipreload = 0
                self.clipleft = self.clipmax
                self.reloading = 0
                
        self.canfire = (self.firereload <= 0) and not(self.reloading)
    
    def fire(self,firing):
        if (firing) and (self.semiauto):
            return 0
        if (self.reloading) or (self.firereload>0):
            return 0
        if self.clipleft == 0:
            self.reloading = 1
            self.clipreload = self.cliptime
            self.reloading = 1
            return 0        
        self.clipleft -= 1
        self.firereload = self.firetime
        return 1
    
class Shotgun(Weapon):
    def __init__(self):
        Weapon.__init__(self,1,0.1,7,0.5)
        
        self.accuracy = 200
        self.shots = 5
        self.bulletspeed = 2000
        self.firing = 0
    
    def shoot(self,position,direction,room):
        if self.fire(self.firing):
            resources.shotgunsound.play()
            
            velocity = direction * self.bulletspeed
            for i in range(1,self.shots):
                scatter = Vector((random.random()-0.5)*self.accuracy,(random.random()-0.5)*self.accuracy)
                room.addprojectile(Projectile(room,position,velocity+scatter))
            self.firing = 1

class Player(GameActor):
    def __init__(self,xpos,ypos,xextents=10,yextents=20):
        GameActor.__init__(self,xpos,ypos,xextents,yextents,isplayer=1) 
        self.currentweapon = Shotgun()
        self.startpos = self.position.copy()
        
    def update(self):
        self.move()
        self.currentweapon.update(manager.frametime)
        
    def controlweapons(self,controls,room):
        if controls.mouseleft:
            direction = controls.mouseworldpos-self.position
            direction.normalise()
            self.currentweapon.shoot(self.position,direction,room)
        else:
            self.currentweapon.firing = not(self.currentweapon.canfire)
        
    def controlmovement(self,controls):
        if controls.keylist[LEFT_KEY].state:
            self.velocity.x = -200
            self.faceleft = 1
        elif controls.keylist[RIGHT_KEY].state:
            self.velocity.x = 200
            self.faceleft = 0
        else:
            self.velocity.x = 0
        if controls.keylist[JUMP_KEY].state:
            self.attemptjump()
        elif self.canjump:
            self.jumping = 0
            
    def die(self):
        self.position = self.startpos.copy()
        self.velocity = Vector()
        
        resources.deathsound.play()

class Spike(Block):
    def __init__(self, tl=Vector(), br=Vector()):
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)
        
        self.colour = (255,0,0)
        self.image = resources.spikeimage
        
    def reactactor(self,actor):
        if actor.isplayer:
            actor.die()
            
class Finish(Block):
    def __init__(self, tl=Vector(), br=Vector()):
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)
        
        self.colour = (255,0,255)
        self.image = resources.finishimage
        
    def reactactor(self,actor):
        if actor.isplayer:
            endgame()
            
class Door(Block):
    def __init__(self, tl=Vector(), br=Vector(), nextroom="", entryid = ""):
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)
        self.nextroom = nextroom
        self.entryid = entryid
        
        self.colour = (0,255,0)
        self.image = resources.doorimage
        
    def makefromstring(self,line):
        line = line[line.find(',')+1:]
        x0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        x1 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y1 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        destination = line[:line.find(',')]
        line = line[line.find(',')+1:]
        entry = line
        
        tl = Vector(x0,y0)
        br = Vector(x1,y1)
        self.customrect = CustomRect((tl+br)/2,(br-tl)/2)
        self.nextroom = destination
        self.entryid = entry
        
    def reactactor(self,actor):
        if actor.isplayer:
            readynextroom(self.nextroom,self.entryid)

class Entry:
    def __init__(self,entryid="",position=Vector()):
        self.entryid = entryid
        self.position = position
        
    def makefromstring(self,line):
        line = line[line.find(',')+1:]
        x0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y0 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        x1 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        y1 = int(line[:line.find(',')])
        line = line[line.find(',')+1:]
        entry = line
        
        tl = Vector(x0,y0)
        br = Vector(x1,y1)
        self.position = (tl+br)/2
        self.entryid = entry

class Room:
    def __init__(self,path):
        self.width = 0
        self.height = 0
        
        self.loadfromfile(path)
        
        self.projectiles = []
        
    def addblock(self,block):
        self.blocks.append(block)
        
    def addprojectile(self,projectile):
        self.projectiles.append(projectile)
        
    def loadfromfile(self,path):
        actualpath = os.path.join('levels',path)
        saveFile = open(actualpath,'r')
        temp = saveFile.readline()
        if temp[:temp.find(',')] != "KRM 1.0":
            print "This KRM loader only accepts KRM 1.0 files"
            return []
        else:
            self.blocks = []
            self.entries = []
            temp = temp[temp.find(',')+1:]
            width = int(temp[:temp.find(',')])
            temp = temp[temp.find(',')+1:]
            height = int(temp[:temp.find(',')])
            temp = temp[temp.find(',')+1:]
            self.width = width
            self.height = height
            temp = saveFile.readline()
            while temp != '':
                typename = temp[:temp.find(',')]
                temp = temp[temp.find(',')+1:]                
                if typename in ("Platform","Door","Spike","Finish"):
                    if typename == "Platform":
                        newblock = Block()
                    if typename == "Door":
                        newblock = Door()
                    if typename == "Spike":
                        newblock = Spike()
                    if typename == "Finish":
                        newblock = Finish()
                    newblock.makefromstring(temp)    
                    self.blocks.append(newblock)
                if typename == "Entry":
                    newentry = Entry()
                    newentry.makefromstring(temp)
                    self.entries.append(newentry)
                temp = saveFile.readline()

class ResourceBucket:
    def __init__(self):
        self.backimage = loadimage('background.png')
        self.blockimage = loadimage('block.png')
        self.doorimage = loadimage('door.png')
        self.spikeimage = loadimage('spike.png')
        self.finishimage = loadimage('finish.png')
        
        self.guyanim = []
        self.guyanim.append(loadimage('guy1.png'))
        self.guyanim.append(loadimage('guy2.png'))
        
        self.jumpsound = loadsound('jump.wav')
        self.deathsound = loadsound('death.wav')
        self.shotgunsound = loadsound('shotgun.wav')
        
        self.music = loadsound('music.ogg')

def endgame():
    maingameloop.endinggame = 1

class QuitLoop(GameLoop):
    def update(self):
        manager.quitting = 1

def readynextroom(nextroom,entryid):
    maingameloop.nextroom = nextroom
    maingameloop.nextentry = entryid
    maingameloop.movingtonew = 1

class MainLoop(GameLoop):
    def __init__(self):
        GameLoop.__init__(self)
        self.playercontrols = Controls([pygame.K_a,pygame.K_d,pygame.K_w])
        manager.addeventlistener(self.playercontrols)  
        self.mousepos = self.playercontrols.mousepos.copy()
        
        self.nextroom = ""
        self.nexentry = ""
        self.movingtonew = 0
        
        self.endinggame = 0
        
        self.player = Player(400,200)
        self.openlevel('start.krm','start')       
        
        self.camera = self.player.position.copy()
        self.lastcamera = self.camera   
        
        resources.music.play(-1)
        
    def controlentities(self):
        self.player.controlmovement(self.playercontrols)
        self.player.controlweapons(self.playercontrols,self.currentroom)
    
    def updateentities(self):
        self.player.update()
        for projectile in self.currentroom.projectiles:
            projectile.move()
    
    def collideentities(self):
        self.player.collide(self.currentroom.blocks)
        self.currentroom.deadbullets = []
        for projectile in self.currentroom.projectiles:
            projectile.collide()
        for bullet in self.currentroom.deadbullets:
            self.currentroom.projectiles.remove(bullet)
    
    def updatescrolling(self):
        halfextents = Vector(manager.width/2,manager.height/2)
        
        mousescreenpos = self.playercontrols.mousepos
        self.camera = self.player.position+(mousescreenpos-halfextents)/2-halfextents
        self.camera = (self.camera+self.lastcamera*10)/11
        self.lastcamera = self.camera.copy()
        
        self.mousepos = self.camera+mousescreenpos
        self.playercontrols.mouseworldpos = self.mousepos
    
    def drawbackground(self):
        manager.screen.fill((255,255,255))
        #manager.screen.blit(resources.backimage[0],resources.backimage[1])
        
    def drawblocks(self):
        drawrect = CustomRect(Vector(manager.width/2-self.camera.x,manager.height/2-self.camera.y),
        Vector(self.currentroom.width,self.currentroom.height)/2)
        imagerect = pygame.Rect(0,0,self.currentroom.width,self.currentroom.height)
        imagepair = (self.blockbuffer,imagerect)
        drawrect.drawimage(manager.screen,imagepair)
            
    def drawplayer(self):
        drawrect = self.player.getrect()
        drawrect.rect = drawrect.rect.move(-self.camera.x,-self.camera.y)
        
        drawimage = resources.guyanim[self.player.currentframe]
        if self.player.faceleft:
            flippedimage = pygame.transform.flip(drawimage[0],1,0)
            drawimage = (flippedimage,drawimage[1])
        #drawrect.draw(manager.screen,(255,0,0))
        drawrect.drawimage(manager.screen,drawimage)
        
    def drawbullets(self):
        for projectile in self.currentroom.projectiles:
            projectile.draw((0,0,0),self.camera)
    

    def drawentities(self):
        #manager.screen.lock()
        self.drawbackground()
        self.drawblocks()
        self.drawplayer()
        self.drawbullets()
        #manager.screen.unlock()
        
    def drawblockbuffer(self):
        BLOCK_TRANS_COLOUR = (128,255,0)
        self.blockbuffer = pygame.Surface((self.currentroom.width,self.currentroom.height))
        self.blockbuffer.fill(BLOCK_TRANS_COLOUR)
        self.blockbuffer.set_colorkey(BLOCK_TRANS_COLOUR)
        for block in self.currentroom.blocks:
            drawrect = block.customrect
            drawrect.drawtiledimage(self.blockbuffer,block.image)
            #drawrect.draw(self.blockbuffer,block.colour)
        
    def openlevel(self,nextlevel,entry):
        self.currentroom = Room(nextlevel)
        for readentry in self.currentroom.entries:
            if readentry.entryid == entry:
                self.player.position = readentry.position.copy()
                self.player.startpos = self.player.position.copy()
                break
        self.drawblockbuffer()
        
    def checknewlevel(self):
        if not(self.movingtonew):
            return
        self.openlevel(self.nextroom,self.nextentry)
        self.movingtonew = 0
        
    def checkgameend(self):
        if self.endinggame:
            self.quitloop()
            endingloop = ImageViewerLoop('end.png',QuitLoop())
            endingloop.run()
        
    def update(self):
        self.checkgameend()
        self.checknewlevel()
        self.controlentities()
        self.updateentities()
        self.collideentities()
        self.updatescrolling()
        self.drawentities()
        

manager = MainManager()
resources = ResourceBucket()
maingameloop = MainLoop()
titleloop = ImageViewerLoop('title.jpg',maingameloop)
introloop = ImageViewerLoop('start.png',titleloop)
introloop.run()
pygame.quit()
pygame.mixer.quit()
