from Tkinter import *
import tkFileDialog, tkSimpleDialog

class MyDialog(tkSimpleDialog.Dialog):
    def body(self, master):

        Label(master, text="Width:").grid(row=0)
        Label(master, text="Height:").grid(row=1)

        self.e1 = Entry(master)
        self.e2 = Entry(master)

        self.e1.grid(row=0, column=1)
        self.e2.grid(row=1, column=1)
        return self.e1 # initial focus

    def apply(self):
        graphicalEditor.width = int(self.e1.get())
        graphicalEditor.height = int(self.e2.get())
        graphicalEditor.newCanvas()

        
class LevelElement:
    def __init__(self,type,x0,y0,x1,y1):
        self.x0 = int(x0)
        self.y0 = int(y0)
        self.x1 = int(x1)
        self.y1 = int(y1)
        self.typename = type

class MenuPanel:
    def __init__(self, master):
        self.menubar = Menu(root)
        mnuFile = Menu(self.menubar, tearoff=0)
        mnuFile.add_command(label="New", command = self.fileNew)
        mnuFile.add_command(label="Open", command = self.fileOpen)
        mnuFile.add_command(label="Save", command = self.fileSave)
        mnuFile.add_command(label="Quit", command = self.fileQuit)
        self.menubar.add_cascade(label="File", menu = mnuFile)

    def fileNew(self):
        D=MyDialog(root)
        templist = graphicalEditor.stuffList
        graphicalEditor.elements = []
        graphicalEditor.stuffList = []
        for block in templist:
            #graphicalEditor.stuffList.remove(block)
            graphicalEditor.cnvEditPad.delete(block)

    def fileSave(self):
        saveFile = tkFileDialog.asksaveasfile(filetypes=[('Killer Robot Map', '*.krm')])
        saveFile.write("KRM 1.0," + str(graphicalEditor.width) + "," + str(graphicalEditor.height) + '\n')
        for temp in graphicalEditor.elements:
            line = temp.typename+ ',' + 'SOLID' + ',' + str(temp.x0) + ',' + str(temp.y0)
            if temp.typename in ("Platform","Door","Entry","Spike","Finish"):
                line = line + ',' + str(temp.x1) + ',' + str(temp.y1)
            if temp.typename == "Door":
                line = line + ',' + temp.nextroom + ',' + temp.entry
            if temp.typename == "Entry":
                line = line + ',' + temp.entry
            line = line + '\n'
            saveFile.write(line)

    def fileOpen(self):
        templist = graphicalEditor.stuffList
        graphicalEditor.elements = []
        graphicalEditor.stuffList = []
        for block in templist:
            #graphicalEditor.stuffList.remove(block)
            graphicalEditor.cnvEditPad.delete(block)
        saveFile = tkFileDialog.askopenfile(filetypes=[('Killer Robot Map', '*.krm')])
        temp = saveFile.readline()
        if temp[:temp.find(',')] != "KRM 1.0":
            print "This KRM editor only accepts KRM 1.0 files"
        else:
            temp = temp[temp.find(',')+1:]
            graphicalEditor.width = int(temp[:temp.find(',')])
            temp = temp[temp.find(',')+1:]
            graphicalEditor.height = int(temp)
            temp = saveFile.readline()
            while temp != '':
                typename = temp[:temp.find(',')]
                temp = temp[temp.find(',')+1:]                
                temp = temp[temp.find(',')+1:]
                x0 = int(temp[:temp.find(',')])
                temp = temp[temp.find(',')+1:]
                y0 = int(temp[:temp.find(',')])
                temp = temp[temp.find(',')+1:]
                x1 = int(temp[:temp.find(',')])
                temp = temp[temp.find(',')+1:]
                y1 = int(temp[:temp.find(',')])
                if typename in ("Door"):
                    temp = temp[temp.find(',')+1:]
                    nextroom = temp[:temp.find(',')]
                if typename in ("Door","Entry"):
                    temp = temp[temp.find(',')+1:]
                    entry = temp[:temp.find(',')]
                tempelement = LevelElement(typename,x0,y0,x1,y1)
                if typename in ("Door"):
                    tempelement.nextroom = nextroom
                if typename in ("Door","Entry"):
                    tempelement.entry = entry
                graphicalEditor.elements.append(tempelement)
                colour = "#FF0000"
                if typename == "Platform":
                    colour = "#FFFFFF"
                if typename == "Door":
                    colour = "#00FF00"
                if typename == "Entry":
                    colour = "#0000FF"
                if typename == "Spike":
                    colour = "#FF0000"
                if typename == "Finish":
                    colour = "#FF00FF"
                temp = graphicalEditor.cnvEditPad.create_rectangle(x0, y0, x1, y1, fill=colour)
                graphicalEditor.stuffList.append(temp)
                temp = saveFile.readline()

    def fileQuit(self):
        frmMain.quit()
        quit()


class ControlPanel:
    def __init__(self, master):
        frmToolbar = Frame(frmMain)
        frmToolbar.pack(side=TOP)    

        btnMove = Button(frmToolbar, text = 'Move', command = self.Move)
        btnMove.pack(side=LEFT)

        btnDelete = Button(frmToolbar, text = 'Delete', command = self.Delete)
        btnDelete.pack(side=LEFT)       
        
        lblRoom = Label(frmToolbar, text="Next Room:")
        lblRoom.pack(side=LEFT)
        
        self.txtRoom = Entry(frmToolbar, width = 10)
        self.txtRoom.pack(side=LEFT)
        
        lblEntry = Label(frmToolbar, text="EntryID:")
        lblEntry.pack(side=LEFT)
        
        self.txtEntry = Entry(frmToolbar, width = 10)
        self.txtEntry.pack(side=LEFT)

    def Move(self):
        GraphicalEditor.mode="Move"

    def Delete(self):
        GraphicalEditor.mode="Delete"    
        
class ItemPanel:
    def __init__(self, master):
        frmItembar = Frame(frmMain)
        frmItembar.pack(side=LEFT)  

        btnPlatform = Button(frmItembar, text = 'Platform', command = self.Platform)
        btnPlatform.pack(side=TOP)

        btnDoor = Button(frmItembar, text = 'Door', command = self.Door)
        btnDoor.pack(side=TOP)
        
        btnEntry = Button(frmItembar, text = 'Entry', command = self.Entry)
        btnEntry.pack(side=TOP)
        
        btnSpike = Button(frmItembar, text = 'Spike', command = self.Spike)
        btnSpike.pack(side=TOP)
        
        btnFinish = Button(frmItembar, text = 'Finish', command = self.Finish)
        btnFinish.pack(side=TOP)
        
    def Platform(self):
        GraphicalEditor.mode="Platform"

    def Door(self):
        GraphicalEditor.mode="Door"
        
    def Entry(self):
        GraphicalEditor.mode="Entry"
        
    def Spike(self):
        GraphicalEditor.mode="Spike"
    
    def Finish(self):
        GraphicalEditor.mode="Finish"
        
class ViewPanel:
    def __init__(self, master):
        frmViewbar = Frame(frmMain)
        frmViewbar.pack(side=BOTTOM)  
        
        self.lblGridsize = Label(frmViewbar, text = "Grid Size: "+str(GraphicalEditor.gridsize)+" ")
        self.lblGridsize.pack(side=LEFT)

        self.btnToggle = Button(frmViewbar, text = 'Disable Grid', command = self.togglegrid)
        self.btnToggle.pack(side=LEFT)

        btnMult = Button(frmViewbar, text = 'Grid*2', command = self.multgrid)
        btnMult.pack(side=LEFT)

        btnDiv = Button(frmViewbar, text = 'Grid/2', command = self.divgrid)
        btnDiv.pack(side=LEFT)

    def togglegrid(self):
        if GraphicalEditor.gridon:
            GraphicalEditor.gridon = 0
            self.btnToggle.config(text = "Enable Grid")
        else:
            GraphicalEditor.gridon = 1
            self.btnToggle.config(text = "Disable Grid")
            

    def multgrid(self):
        GraphicalEditor.gridsize *= 2
        self.lblGridsize.config(text = "Grid Size: "+str(GraphicalEditor.gridsize)+" ")

    def divgrid(self):
        GraphicalEditor.gridsize= GraphicalEditor.gridsize // 2
        if GraphicalEditor.gridsize < 1:
            GraphicalEditor.gridsize = 1
        self.lblGridsize.config(text = "Grid Size: "+str(GraphicalEditor.gridsize)+" ")

    
class GraphicalEditor:
    x0 = 0
    y0 = 0
    x1 = 0
    y1 = 0
    width = 640
    height = 480
    temp = 0
    stuffList = []
    mode = "Move"
    gridsize = 8
    gridon = 1
    
    def __init__(self,master):
        self.cnvEditPad = Canvas(frmMain, width = self.width, height = self.height, bg = "#000000")
        self.cnvEditPad.pack(side=RIGHT)
        self.cnvEditPad.bind("<ButtonPress-1>", self.mouseB1Down)
        self.cnvEditPad.bind("<ButtonRelease-1>", self.mouseB1Up)
        self.cnvEditPad.bind("<B1-Motion>", self.mouseB1Move)

        self.elements = []

    def newCanvas(self):
        self.cnvEditPad.destroy()
#        if (self.width > 640 and self.height > 480):
#            self.cnvEditPad = Canvas(frmMain, width = 640, height = 480, bg = "#000000", scrollregion = (0,0,self.width, self.height), xscrollcommand=True, yscrollcommand=False)
#        else:
        self.cnvEditPad = Canvas(frmMain, width = self.width, height = self.height, bg = "#000000")
        self.cnvEditPad.pack(side=RIGHT)
        self.cnvEditPad.bind("<ButtonPress-1>", self.mouseB1Down)
        self.cnvEditPad.bind("<ButtonRelease-1>", self.mouseB1Up)
        self.cnvEditPad.bind("<B1-Motion>", self.mouseB1Move)
        x0 = 0
        y0 = 0
        x1 = 0
        y1 = 0
        temp = 0
        stuffList = []
        mode = "Move"
        gridsize = 8
        gridon = 1

    def mouseB1Down(self,event):
        self.x0 = round(event.x/float(self.gridsize))*self.gridsize #+ event.x%self.gridsize/4
        self.y0 = round(event.y/float(self.gridsize))*self.gridsize #+ event.y%self.gridsize/4
        if self.mode == "Delete":
            for element in self.elements:
                if (event.x > element.x0) and (event.x < element.x1):
                    if (event.y > element.y0) and (event.y < element.y1):
                        self.elements.remove(element)
            for block in self.stuffList:
                bounds = self.cnvEditPad.bbox(block)
                if ((event.x > bounds[0] and event.x < bounds[2]) or (event.x > bounds[2] and event.x < bounds[0])):
                    if ((event.y > bounds[1] and event.y < bounds[3]) or (event.y > bounds[3] and event.y < bounds[1])):
                        self.cnvEditPad.delete(block)
                        self.stuffList.remove(block)

    def mouseB3Down(self,event):
        tempList = self.stuffList
        for block in tempList:
            bounds = self.cnvEditPad.bbox(block)
            if ((event.x > bounds[0] and event.x < bounds[2]) or (event.x > bounds[2] and event.x < bounds[0])):
                if ((event.y > bounds[1] and event.y < bounds[3]) or (event.y > bounds[3] and event.y < bounds[1])):
                    self.cnvEditPad.delete(block)
                    self.stuffList.remove(block)

    def mouseB1Up(self,event):
        colour = "#FF0000"
        if self.mode in ("Platform","Door","Entry","Spike","Finish"):            
            if self.mode == "Platform":
                colour = "#FFFFFF"
            if self.mode == "Door":
                colour = "#00FF00"
            if self.mode == "Entry":
                colour = "#0000FF"
            if self.mode == "Spike":
                colour = "#FF0000"
            if self.mode == "Finish":
                colour = "#FF00FF"
            temp = self.cnvEditPad.create_rectangle(self.x0, self.y0, self.x1, self.y1, fill=colour)
            self.stuffList.append(temp)
            tlx = min(self.x0,self.x1)
            tly = min(self.y0,self.y1)
            brx = max(self.x0,self.x1)
            bry = max(self.y0,self.y1)
            tempelement = LevelElement(self.mode,tlx,tly,brx,bry)
            if self.mode == "Door":
                tempelement.nextroom = controlPanel.txtRoom.get()
                tempelement.entry = controlPanel.txtEntry.get()
            if self.mode == "Entry":
                tempelement.entry = controlPanel.txtEntry.get()
            self.elements.append(tempelement)
            self.cnvEditPad.delete(self.temp)

    def mouseB1Move(self,event):
        if self.mode in ("Platform","Door","Entry","Spike","Finish"):
            self.cnvEditPad.delete(self.temp)
            self.x1 = round(event.x/float(self.gridsize))*self.gridsize #+ event.x%self.gridsize/4
            self.y1 = round(event.y/float(self.gridsize))*self.gridsize #+ event.y%self.gridsize/4
            tlx = min(self.x0,self.x1)
            tly = min(self.y0,self.y1)
            brx = max(self.x0,self.x1)
            bry = max(self.y0,self.y1)
            self.temp = self.cnvEditPad.create_rectangle(tlx, tly, brx, bry, fill="#7F7F7F")
                

        
root = Tk()
frmMain = Frame(root)
frmMain.pack()
menuPanel = MenuPanel(root)
controlPanel = ControlPanel(root)
itemPanel = ItemPanel(root)
viewPanel = ViewPanel(root)
graphicalEditor = GraphicalEditor(root)
root.config(menu=menuPanel.menubar)
root.mainloop()
